#include <windows.h>
#include <dxerr9.h>
#include <dsound.h>
#include <string.h>

#include "dsutil.h"

#include <string>
using namespace std;


// Window functions and variables
HINSTANCE g_hInst;				// holds the instance for this app
HWND g_wndHandle;					// global window handle

bool    initWindow(HINSTANCE hInstance);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

// DirectSound functions and variables
CSoundManager*	g_pSoundManager = NULL;
CSound*			g_pSound		= NULL;
CSound*			g_pClickSound	= NULL;
BOOL			g_bBufferPaused;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	HRESULT hr;
	// call our function to init and create our window
	if (!initWindow(hInstance))
	{
		MessageBox(NULL, "Unable to create window", "WinMain", MB_OK);
		return false;
	}

	g_pSoundManager	= new CSoundManager();
	if(NULL == g_pSoundManager)
	{
		DXTRACE_ERR_MSGBOX(TEXT("Initialize"), E_OUTOFMEMORY);
		MessageBox(NULL, "Unable to create a directsound driver", "WinMain", MB_OK);
		return false;
	}
	if(FAILED(hr = g_pSoundManager->Initialize(g_wndHandle, DSSCL_PRIORITY)))
	{
		DXTRACE_ERR_MSGBOX(TEXT("Initialize"), hr);
		MessageBox(NULL, "Can not initialize a directsound driver", "WinMain", MB_OK);
		return false;
	}

	if(FAILED(hr = g_pSoundManager->SetPrimaryBufferFormat(2, 22050, 16)))
	{
		DXTRACE_ERR_MSGBOX(TEXT("SetPrimaryBufferFormat"), hr);
		MessageBox(NULL, "Can not create primary buffer for a directsound", "WinMain", MB_OK);
		return false;
	}

	g_bBufferPaused = FALSE;

	if(FAILED(hr = g_pSoundManager->Create(&g_pSound, "sound1.wav", 0, GUID_NULL)))
	{
//		DXTRACE_ERR( TEXT("Loading wave file!"));
		MessageBox(NULL, "Can not load wave file", "WinMain", MB_OK);
	}
	if(FAILED(hr = g_pSoundManager->Create(&g_pClickSound, "click.wav", 0, GUID_NULL)))
	{
//		DXTRACE_ERR( TEXT("Loading wave file!"));
		MessageBox(NULL, "Can not load wave file", "WinMain", MB_OK);
	}

	// Main message loop:
	// Enter the message loop
    MSG msg; 
    ZeroMemory( &msg, sizeof(msg) );
    while( msg.message!=WM_QUIT )
    {
		// check for messages
		if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
        {
			TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
		else
		{
			if( g_pSound->IsSoundPlaying() )
			{
				
			}
			else
			{
				// The buffer is not playing, so play it again
				if( FAILED( hr = g_pSound->Play( 0, DSBPLAY_LOOPING ) ) )
					return DXTRACE_ERR( TEXT("Play"), hr );
				if (FAILED(hr = g_pClickSound->Play(0, DSBPLAY_LOOPING)))
					return DXTRACE_ERR( TEXT("Play"), hr );			
			}
		}
	}
	delete g_pSound;
	delete g_pClickSound;
	delete g_pSoundManager;
	return (int) msg.wParam;
}


/*******************************************************************
* WndProc
* The window procedure for this window
*******************************************************************/
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_LBUTTONDOWN:
			g_pClickSound->Play();
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}


/*******************************************************************
* initWindow
* inits and creates the window
*******************************************************************/
bool initWindow(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= "DirectXExample";
	wcex.hIconSm		= 0;
	RegisterClassEx(&wcex);

	g_wndHandle = CreateWindow("DirectXExample", 
							 "DirectXExample", 
							 WS_OVERLAPPEDWINDOW,
							 CW_USEDEFAULT, 
							 CW_USEDEFAULT, 
							 640, 
							 480, 
							 NULL, 
							 NULL, 
							 hInstance, 
							 NULL);
   if (!g_wndHandle)
      return false;
   
   ShowWindow(g_wndHandle, SW_SHOW);
   UpdateWindow(g_wndHandle);

   return true;
}
